/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "CMYLookup.h"
#include "RGB.h"
#include "errors.h"
#include "CMYData.h"

CMYLookup::CMYLookup()
{
	xlen = ylen = zlen = 16;
	xyz = cmy;
}

CMYLookup::~CMYLookup()
{
	xyz = 0;
}

void CMYLookup::touchDataLines(Map *imageMap, int start,int len,void *ptr)
{
	for(int y = 0; y < len; y++)
	{
		for(int x = 0; x < imageMap->width; x++)
		{
			//convert RGB to CMY
			RGB *rgb = &((RGB *)ptr)[y*imageMap->width + x];

			// since so much of ghostscript images are white, we want to return really fast
			if(rgb->r == 255 && rgb->g == 255 && rgb->b == 255)
			{
				rgb->r = rgb->g = rgb->b = 0;
				continue;
			}
			

			//create a nice little cube of points
			int pos1 = (rgb->r * (xlen - 1) / 256 * ylen * zlen + rgb->g * (ylen - 1)  / 256 * zlen + rgb->b * (zlen - 1) / 256) * 3;
			int pos2 = pos1 + 3; // shifted in z direction
			int pos3 = pos1 + 3 * zlen; // y
			int pos4 = pos1 + 3 * zlen + 3; // y+z
			int pos5 = pos1 + 3 * ylen * zlen; // x
			int pos6 = pos1 + 3 * ylen * zlen + 3; // x+z
			int pos7 = pos1 + 3 * ylen * zlen + 3 * zlen; // x + y
			int pos8 = pos1 + 3 * ylen * zlen + 3 * zlen + 3; // x + y + z

			//now compute the x y z percentages of where the dot actually is between the positions
			int xp = rgb->r % xlen;
			int yp = rgb->g % ylen;
			int zp = rgb->b % zlen;

			//multiply each pos by the total offset from it to get a final averaged answer
			long new_r = ((long)xyz[pos8] * xp * yp * zp + 
						 xyz[pos7] * xp * yp * (zlen-zp) +
						 xyz[pos6] * xp * (ylen-yp) * zp +
						 xyz[pos5] * xp * (ylen-yp) * (zlen-zp) +
						 xyz[pos4] * (xlen-xp) * yp * zp +
						 xyz[pos3] * (xlen-xp) * yp * (zlen-zp) +
						 xyz[pos2] * (xlen-xp) * (ylen-yp) * zp +
						 xyz[pos1] * (xlen-xp) * (ylen-yp) * (zlen-zp)) / xlen / ylen / zlen;
			long new_b = ((long)xyz[pos8+1] * xp * yp * zp + 
						 xyz[pos7+1] * xp * yp * (zlen-zp) +
						 xyz[pos6+1] * xp * (ylen-yp) * zp +
						 xyz[pos5+1] * xp * (ylen-yp) * (zlen-zp) +
						 xyz[pos4+1] * (xlen-xp) * yp * zp +
						 xyz[pos3+1] * (xlen-xp) * yp * (zlen-zp) +
						 xyz[pos2+1] * (xlen-xp) * (ylen-yp) * zp +
						 xyz[pos1+1] * (xlen-xp) * (ylen-yp) * (zlen-zp)) / xlen / ylen / zlen;
			long new_g = ((long)xyz[pos8+2] * xp * yp * zp + 
						 xyz[pos7+2] * xp * yp * (zlen-zp) +
						 xyz[pos6+2] * xp * (ylen-yp) * zp +
						 xyz[pos5+2] * xp * (ylen-yp) * (zlen-zp) +
						 xyz[pos4+2] * (xlen-xp) * yp * zp +
						 xyz[pos3+2] * (xlen-xp) * yp * (zlen-zp) +
						 xyz[pos2+2] * (xlen-xp) * (ylen-yp) * zp +
						 xyz[pos1+2] * (xlen-xp) * (ylen-yp) * (zlen-zp)) / xlen / ylen / zlen;

			
			
			//fprintf(stderr,"%3d %3d %3d->%3ld %3ld %3ld xp %3d yp %3d zp %3d\n",rgb->r,rgb->b,rgb->g,new_r,new_g,new_b,xp,yp,zp);

			rgb->r = new_r;
			rgb->b = new_g;
			rgb->g = new_b;
			
		}
	}
}






