/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef JET_H
#define JET_H

#include "Map.h"
#include "Coord.h"
#include "Rect.h"
#include "LexmarkProtocol.h"

//jet marker indicating left jet
#define LEFT_JET 0
//jet marker indicating right jet
#define RIGHT_JET 1

class Jet
{
 private:
	int id,     // id of jet
 		x,y,    // x and y position from head x,y
		side,   // either LEFT_JET or RIGHT_JET
		color,  // color of jet (0,1,2) == (r,g,b)
		ystart; // start of dots needed to be painted from Image coordinates

	int dots; // number of dots in this jet(does not consider the dots of it's partner)
	
	Map *imageMap;
	Coord imageStart;  // start of image from edge of paper

	bool insideImage(int x, int y);
	bool needsPainting(int ix, int iy, int bit);
	
 public:
	/* Range of paper jet covers when head is at 0,0 */
	void getRange(Rect *dest);

	/* Range of paper jet needs to cover to get the entire image */
	void getImageRange(Rect *dest);
	Coord *getPreferredImageStart(Coord *start);
	int getPreferredEnd();
	bool finished();
	void setImageStart(Coord *imageStart);
	void paint(int startX, int endX, int y, class LexmarkProtocol *protocol);

	/* returns true if the jet needs to paint on the given y line. */
	bool needsCover(int y);

	/* finds the travel of the head needed to cover all the dots. Returns false if no dots needed cover */
	bool findStartEnd(int y, int *startX, int *endX);
	
	
	Jet(int id, int x, int y, Map *imageMap, int side, int color, int dots);
};

#endif
