/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <string.h>
#include "errors.h"
#include "PBMRaw.h"
#include "RGB.h"


PBMRaw::PBMRaw(Reader *reader, bool readMagic)
{
	this->reader = reader;
	this->start = 0;
	this->end = 0;
	this->width = -1;
	this->height = -1;
	this->bytes = NULL;
	
	readHeader(readMagic);
}

void PBMRaw::getRGB(RGB *rgb, int x, int y)
{
    int bytex = x / 8;
    int bitmask = 1 << (7 - x % 8);

	int pos = bytex + y * this->byte_width - this->start;

    if(pos < 0 || pos > this->end)
    {
		die("Illegal RGB position, not cached (%d,%d)",x,y);
    }

    //this is a monochrome bitmap so it will be either 255 or 0
    if(this->bytes[pos] & bitmask)
    {
		rgb->r = rgb->g = rgb->b = 255;
		return;
    }

		rgb->r = rgb->g = rgb->b = 0;
		return;
}

void PBMRaw::readHeader(bool readMagic)
{
    //read magic number
	char line[256];
	
	if(readMagic)
	{
		this->reader->readLine(line,255);
		if(PBMRaw::isMagic(line) != 0)
		{
			die("Bad magic number in image file, got %s", line);
		}
	}

    //get width and height
    while(this->reader->readLine(line,255) != -1)
	{
		if(line[0] == '#')
		{
			continue;
		}
		
		if ( sscanf(line,"%d %d",&this->width, &this->height) == 2)
		{
			this->byte_width = (this->width+7)/8; //width in bytes,rounded upwards to the nearest byte
			return; //read width and height, done processing header
		}
		else
		{
			die("Not understood line in image file, %s", line);
		}
    }
		
	die("Got eof looking for width and height");
}

bool PBMRaw::isMagic(char *line)
{
	if(strcmp(line,"P6") == 0)
		return true;
	
	return false;
}

static void test(char *filename)
{
	int old_j, j,x,y;
    FILE *fd;
	Image *image;

	fd = fopen(filename,"r");

	if(fd == NULL)
		die("Couldn't open %s for reading",filename);

	Reader r(fd,true);

	PBMRaw pbmraw(&r,true);

	image = (Image *)&pbmraw;
    
	old_j=0;

    for(j = 1; j <= image->height; j += 1)
    {
		image->cacheLines(old_j,j+1);

		for(y = old_j; y < j;y ++)
		{
			for(x = 0; x < image->width; x++)
			{
				RGB rgb;

				image->getRGB(&rgb,x,y);
				
				if(rgb.r == 1)
				{
					putc('X',stdout);
				}
				else
				{
					putc(' ',stdout);
				}
			}
			old_j = j;
			putc('\n',stdout);
		}
	}
	
}


#ifdef PBM_TEST

int main(int argc, char **argv)
{
	if(argc != 2)
	{
		die("Usage: %s <pbm file>\n",argv[0]);
	}
	
	test(argv[1]);
	
}

#endif
