/*
Lexmark 1100 printer driver for Linux
Copyright (C) 1999   Tim Engler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdlib.h>
#include <string.h>
#include "Head.h"
#include "LexmarkProtocol.h"
#include "LexmarkProtocolBW.h"
#include "LexmarkProtocol300c.h"
#include "PPMRawAlg.h"
#include "ErrorDiffusionAlg.h"
#include "CMYLookup.h"
#include "CMYAlg.h"
#include "errors.h"
#include "RGB.h"
#include "SpongeWriter.h"
#include "PrinterProtocol.h"

#define SPONGE_BUF 512000

static void print(FILE *fp, SpongeWriter *sp, bool blackwhite, int brightness);


int main(int argc, char *argv[])
{
	if(argc < 2 || argc >5)
	{
		die("Usage: %s [-bw] [-b value] <ppm/pbm raw file|->\n   -bw black and white cartridge printing\n   -b brightness value 0-255 defaults to 255. Less brightness = more ink\n",argv[0]);
	}

	bool blackwhite = false;
	int brightness = 255;
	int file_index = 0;

	while(++file_index < argc -1)
	{
		if(strcmp(argv[file_index],"-bw") == 0)
			blackwhite = true;
		else if(strcmp(argv[file_index],"-b") == 0)
			brightness = atoi(argv[++file_index]);
	}
	
	SpongeWriter sp(SPONGE_BUF,1);

	if(strcmp(argv[file_index],"-") != 0) 
	{
		FILE *fd;
		
		fd = fopen(argv[file_index],"r");
		
		if(fd == NULL)
			die("Couldn't open %s for reading\n",argv[file_index]);

		print(fd,&sp,blackwhite,brightness);
	}
	else {
		// reading from stdin, accept as many images that are sent
		while(1)
		{
			int c = getc(stdin);
			if(c == EOF)
				break;
			ungetc(c,stdin);
			
			print(stdin,&sp,blackwhite,brightness);
		}
	}
}

void print(FILE *fd,SpongeWriter *sp,bool blackwhite, int brightness)
{
	Reader r(fd,false);
	char buffer[255];
	
	PPMRawAlg alg(&r, true);
	Map m(alg.width,alg.height,sizeof(RGB));
	m.addAlgorithm(&alg);
	ErrorDiffusionAlg e(brightness);
	CMYLookup c;
	
	m.addAlgorithm(&c);
	m.addAlgorithm(&e);
	
	LexmarkProtocol *p;
	
	if(blackwhite)
		p = new LexmarkProtocolBW(sp);
	else
		p = new LexmarkProtocol300c(sp);

	//get the page size
	Rect pageSize = p->getPageSize(LETTER_PAGE);
	
	Head h(&m,p, &pageSize);

	h.paint();

	//skip to the end of the image if the head couldn't paint it all
	alg.skipToEnd();

	delete p;
}


